/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSGROUP_H
#define PAPYRUSGROUP_H

#include <list>
#include <map>
#include <vector>

#include <papyrus/drawable.h>

/**
 * \page guide_group Groups
 *
 * A group is a container for ordered collections of shapes, with the shapes
 * at the top (front of the container) drawn last and the shapes at the
 * bottom drawn first (end of the container).
 *
 * The current implementation is very simplistic, and doesn't even try to
 * do anything smart like taking into account occlusion or anything else. Instead
 * all objects are simply drawn and it is left to cairo to take care of
 * efficiency. At the current time there are no plans to improve this behavior,
 * since it runs 'fast enough' for now.
 *
 * Groups are also useful since transforms upon the group are also applied
 * to the contained objects. Thus, scaling, translating or rotating a group
 * has the same effect as performing those operations upon each member of
 * the group.
 *
 * The following contains an example that adds several shapes to a group,
 * then rotates the entire group by 45 degrees counter-clockwise.
 *
 * \image html group.png "Group and shapes drawn by example code below"
 *
 * \dontinclude factory.cpp
 * \skip example_group
 * \until }
 *
 */

namespace Papyrus
{

  /**
   * A group is a container for ordered collections of shapes, with the shapes
   * at the top (front of the container) drawn last and the shapes at the
   * bottom drawn first (end of the container).
   *
   * The current implementation is very simplistic, and doesn't even try to
   * do anything smart like taking into account occlusion or anything else. Instead
   * all objects are simply drawn and it is left to cairo to take care of
   * efficiency. At the current time there are no plans to improve this behavior,
   * since it runs 'fast enough' for now.
   *
   * Groups are also useful since transforms upon the group are also applied
   * to the contained objects. Thus, scaling, translating or rotating a group
   * has the same effect as performing those operations upon each member of
   * the group.
   *
   * \image html group.png "Group contain an ordered set of shapes"
   *
   *
   * @author Rick L Vinyard Jr
   */
  class Group : public Drawable
  {
    protected:

      Group();

    public:
      typedef PapyrusSmartPointer<Group> pointer;

      static pointer create();

      /**
       * A layer is a list of child items.
       * The children are drawn starting with the last item in the
       * list, proceeding forward to the first item. Thus, the first
       * item in the list will be drawn on top of all 'lower' items.
       */
      typedef std::list<Drawable::pointer> Layer;

      /**
       * Layers are drawn starting for the lowest numbered
       * layer to the highest layer.
       */
      typedef std::map<int,Layer> Layers;

      friend class Canvas;

      virtual ~Group();

      /**
       * Adds a child to a given layer.
       * Layer 0 is the default layer.
       * Layers are drawn from the most negative to the most positive. Thus,
       * layer 0 is on top of layer -1, and layer 1 is on top of layer 0.
       */
      virtual bool add( Drawable::pointer item, int layer=0, bool drawing_only=false );

      /**
       * Removes the object from the group.
       */
      virtual bool remove( Drawable::pointer object );

      /**
       * The main purpose of this method is so that a child may use the
       * this pointer to remove itself on destruction.
       */
      virtual bool remove( Drawable* object );

      /**
       * Clears all layers and children from the group.
       */
      virtual bool clear();

      /**
       * Raises the item up n steps in its layer.
       * If steps < 0, the effect is a lower operation.
       */
      virtual bool raise( Drawable::pointer item, int steps=1 );

      /** Raises the item up to the top of its layer */
      virtual bool raise_to_top( Drawable::pointer item );

      /**
       * Lowers the item down n steps in its layer.
       * If steps < 0, the effect is a raise operation.
       */
      virtual bool lower( Drawable::pointer item, int steps=1 );
      
      /**
       * Lowers the item up to the bottom of its layer.
       * Returns true if the item is lowered or already at the bottom,
       * and false if the item cannot be found.
       */
      virtual bool lower_to_bottom( Drawable::pointer item );

      /**
       * Moves an item to a specific layer.
       * The item must already be in this group or no action will be taken.
       */
      virtual bool move_to_layer( int layer, Drawable::pointer item );

      /** Returns the number of children in the group */
      size_t size() const;
      
      /** Returns the layers in this group. */
      const Layers& layers() const;

      /**
       * Returns the requested layer l or throws std::out_of_range
       * if the layer doesn't exist.
       *
       * To avoid throwing an error, check has_layer() before calling.
       */
      const Layer& layer(int l) const throw (std::out_of_range);

      /** True if this group has layer l. */
      bool has_layer( int l );

      /**
       * Overrides parent method for determining whether (x,y) is inside the drawable.
       * If the group itself is selectable, performs a simple extents test.
       * If the group is not selectable, performs a test on each of the children, returning
       * true if any of the children are selectable.
       */
      virtual bool inside( double x, double y );

      virtual std::vector<Drawable::pointer> select( double x, double y, unsigned depth = 1 );

      sigc::signal<void, Drawable::pointer>& signal_child_added();

      sigc::signal<void, Drawable::pointer>& signal_child_removed();

      virtual bool is_group();

      PAPYRUS_CLASS_NAME("Group");

      PAPYRUS_CLONE_METHOD( Group );

    protected:

      // maintaining both items as list for drawing order and connections for quick access
      Layers m_layers;
      size_t m_size;

      typedef std::map<Drawable::pointer, sigc::connection> Connections;
      Connections m_redraw_connections;
      Connections m_changed_connections;

      sigc::signal<void, Drawable::pointer> m_signal_child_added;
      sigc::signal<void, Drawable::pointer> m_signal_child_removed;

      /**
       * Virtual method called when a child is changed.
       * Parameters indicate which child changed and contain hint as to which aspect
       * of the child changed.
       *
       * When a child is added, the child's changed signal is actually connected to the
       * private proxy method, which in turn calls the virtual method. This allows children
       * to implement their own behavior by reimplementing this method without worrying about
       * modifying the add() or remove() methods.
       */
      virtual void on_child_changed( Drawable::pointer child );

      /**
       * Virtual method that recalculates the extents and emits the changed signal.
       *
       * Children should reimplement this method if they need to perform
       * special behavior when an event occurs that could effect the
       * extents, and that behavior needs to take place before the
       * signal is emitted.
       */
      virtual void update_extents();

      void on_child_need_redraw( double x, double y, double w, double h, Drawable::pointer child );
      virtual void draw( Cairo::RefPtr<Cairo::Context> cairo );

      virtual void rebuild_extents();

  };

}

#endif
